__copyright__ = "Copyright (C) 2021  Stefano Zacchiroli"
__license__ = "GPL-3.0-or-later"

# import logging
import magic
import os

from pathlib import Path
from typing import Tuple

from tqdm import tqdm


mime_guesser = magic.Magic(mime=True, mime_encoding=True)


# XXX in wait of Python 3.9 for PER 616...
def _removeprefix(s, prefix):
    if s.startswith(prefix):
        return s[len(prefix):]


def guess_mime(path) -> Tuple[str, str]:
    global mime_guesser

    info = mime_guesser.from_file(path)
    mime_type, encoding = info.split()
    mime_type, encoding = mime_type.rstrip(";"), _removeprefix(encoding, "charset=")

    return (mime_type, encoding)


def iter_blobs(blob_dir: Path):
    # heuristic: if blob_dir/ contains at least two top-level dirs matching "??" we
    # assume the corpus is sharded
    sharded = len([d for d in blob_dir.glob("??") if d.is_dir()]) >= 2

    if sharded:
        # we glob only 2 levels of dirs (max 65k) to avoid sorting all names at once
        for shard in tqdm(sorted(blob_dir.glob("*/*"))):  # level-2 shard dirs
            # logging.info(f"processing shard {shard1.name}")
            for path in sorted(shard.glob("*")):  # individual license blobs
                yield path
    else:
        for root, dirs, files in os.walk(blob_dir):
            root = Path(root)
            for name in files:
                yield root / name
